/*
 *  Compile script for iMatix sources, for OS/2
 *
 *  This script will compile, link, or archive, one or more files.
 *  The syntax is a subset of the syntax of the c bourne shell script.
 *
 *  Written:    98/01/02  Ewen McNeill <ewen@imatix.com>
 *  Revised:    98/10/05  Ewen McNeill <ewen@imatix.com>
 *
 *  Syntax:     c filename...     Compile ANSI C program(s), add to library
 *              c -c filename...  Compile ANSI C program(s)
 *              c -l main         Compile and link main program(s)
 *              c -L main         Link main program(s), no compile
 *              c -r lib file     Replace file.o into library
 *              c -C              Print out full call to compiler
 *
 *                -v              (First arg prefix to above): be verbose
 *                -q              (First arg prefix to above): be quiet
 *
 *  Copyright:  Copyright (c) 1998-99 iMatix Corporation
 *
 *  Bug reports & questions to <ewen@imatix.com>
 *
 *  The following environment variables control its operation:
 *
 *  CCNAME      Name of C compiler (default: gcc)
 *  CCDEFINES   Definitions to pass to compiler (default: -DDEBUG,
 *                   unless CCPRODLEVEL=production)
 *  CCPRODLEVEL One of: debug, standard (default), production
 *              Controls amount of debugging support built in;
 *              debug:       -DEBUG -g
 *              standard:    -DEBUG -O2
 *              production:  -s -O2 -Zomf -Zcrtdll
 *  CCLINBNAME  Library name to archive into (by default; if not
 *                  set arhives into first library found)
 *  ---------------------------------------------------------------
 */
'@echo off'
setlocal

/* Configuration options */

/* Compiler defaults to gcc */
CC = value('CCNAME', '', 'OS2ENVIRONMENT')
IF CC = '' THEN
   CC = 'gcc'

/* Production level defaults to standard */
CCPRODLEVEL = value('CCPRODLEVEL', '', 'OS2ENVIRONMENT')
IF CCPRODLEVEL = '' THEN
   CCPRODLEVEL = 'standard'

/* Compiler definitions defaults to -DDEBUG unless production compile */
CCDEFINES    = value('CCDEFINES', '', 'OS2ENVIRONMENT')
IF CCDEFINES = '' THEN
   IF CCPRODLEVEL = 'production' THEN
      CCDEFINES = ''
   ELSE
      CCDEFINES = '-DDEBUG'

/* Standard flags; use -Zexe so that we can use unix style makefiles.   */
/* sysv-signals are used because iMatix code historically depends on it */
CCFLAGS   = '-Wall -pedantic -Zsysv-signals -Zexe'

/* Other useful arguments; here so we can override them below if required */
LIBRARIES   = '-llibsfl -lsocket'
ARCHIVE     = 'ar -rs'
EOBJ        = '.o'
ELIB        = '.a'

/* Now set up the CCOPTS based on production level, etc */
IF CCPRODLEVEL = 'debug' THEN
DO
   CCOPTS   = CCDEFINES' 'CCFLAGS' -g'       /* Debug: -g */
   LINKOPTS = CCFLAGS' -g'
END 
ELSE IF CCPRODLEVEL = 'production' THEN
DO                                           /* Production: -Zomf -Zcrtdll */
   CCOPTS   = CCDEFINES' 'CCFLAGS' -O2 -Zomf -Zcrtdll' 
   LINKOPTS = CCFLAGS' -s -Zomf -Zcrtdll'
   ARCHIVE  = 'emxomfar rs'
   EOBJ     = '.obj'
   ELIB     = '.lib'
END 
ELSE 
DO
   CCOPTS   = CCDEFINES' 'CCFLAGS' -O2'      /* Standard: -O2 */
   LINKOPTS = CCFLAGS
END

/* Upper case version of obj extension and lib extension */
EOBJU       = TRANSLATE(EOBJ)
ELIBU       = TRANSLATE(ELIB)
OBJLEN      = LENGTH(EOBJ)
LIBLEN      = LENGTH(ELIB)

/* SAY "Compile with: "CC
 * SAY "CCOPTS:       "CCOPTS
 * SAY "LINKOPTS:     "LINKOPTS
 * SAY "LIB = "ELIB"; OBJ = "EOBJ
 * EXIT
 */

/* ------------------------------------------------------------------------ */
/* Pull in RexxUtil library, so that we can can use RxSysFileTree           */
IF RxFuncQuery('SysLoadFuncs') THEN
DO
  IF RxFuncAdd('SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs') THEN
  DO
    SAY "Cannot load RexxUtil (required for RxSysFileTree)"
    EXIT 1
  END
  CALL SysLoadFuncs
END

PARSE ARG commandline
IF commandline == "" THEN
DO
  SAY "Syntax:     c filename...     Compile ANSI C program(s), add to library"
  SAY "            c -c filename...  Compile ANSI C program(s)"
  SAY "            c -l main         Compile and link main program(s)"
  SAY "            c -L main         Link main program(s), no compile"
  SAY "            c -r lib file     Replace object file into library"
  SAY "            c -C              Print out full call to compiler"
  SAY ""
  SAY "              -v              (First arg prefix to above): be verbose"
  SAY "              -q              (First arg prefix to above): be quiet"
  endlocal
  exit 1
END

/* Default is to compile, add to library, but not link */
first   = 1
compile = 1
link    = 0
addtolib= 1
verbose = 1

DO WHILE commandline \= ""
  PARSE VAR commandline thisarg commandline        /* Get next argument */

  IF (first = 1) & (compare(thisarg, "-v") = 0) THEN 
  DO
    verbose = 1
    PARSE VAR commandline thisarg commandline      /* Get next argument */
  END
  ELSE IF (first = 1) & (compare(thisarg, "-q") = 0) THEN 
  DO
    verbose = 0
    PARSE VAR commandline thisarg commandline      /* Get next argument */
  END

  IF (first = 1) & (compare(thisarg, "-c") = 0) THEN
  DO
    compile = 1
    link    = 0
    addtolib= 0
    PARSE VAR commandline thisarg commandline      /* Get next argument */
  END
  ELSE IF (first = 1) & (compare(thisarg, "-l") = 0) THEN
  DO
    compile = 1
    link    = 1
    addtolib= 0
    PARSE VAR commandline thisarg commandline      /* Get next argument */
  END
  ELSE IF (first = 1) & (compare(thisarg, "-L") = 0) THEN
  DO
    compile = 0
    link    = 1
    addtolib= 0
    PARSE VAR commandline thisarg commandline      /* Get next argument */
  END
  ELSE IF (first = 1) & (compare(thisarg, "-r") = 0) THEN
  DO
    compile = 0
    link    = 0
    addtolib= 1
    PARSE VAR commandline library thisarg commandline  /* Get next arg */
  END
  ELSE IF (first = 1) & (compare(thisarg, "-C") = 0) THEN
  DO
    /* Report on the command line invocation, and exit        */
    /* We add the -D__EMX__ bit for the benefit of makedepend */
    /* And we pick up the C_INCLUDE_PATH item if defined and  */
    /* stick that on the end.                                 */
    includepath = value('C_INCLUDE_PATH',,'OS2ENVIRONMENT')
    if (includepath \= '') then includepath = "-I"includepath

    SAY CC "-c" CCOPTS "-D__EMX__" includepath
    exit 0
  END
  ELSE
  DO
    /* Default is to compile and add to local library */
    compile = 1
    link    = 0
    addtolib= 1
    library = FindLocalLib()           /* Determine local library file */
  END

  first = 0

  IF thisarg \= "" THEN
  DO
    IF compile = 1 THEN 
    DO
      IF addtolib = 0 THEN CALL CompileToObj thisarg
                      ELSE CALL CompileToLib thisarg library
    END
    ELSE
    DO
      IF addtolib = 1 THEN CALL ArchiveObj   thisarg  library
    END

    IF link       = 1 THEN CALL LinkProgram  thisarg
  END
END

endlocal
exit 0
/* End script */

/* Utility subroutines */

/* AssertFileExists <file>
 *
 * Checks to see if the specified file (argument 1) exists.  If it does the
 * subroutine returns.  If it doesn't, the subroutine displays an error
 * message, and exits the procedure.
 */

AssertFileExists:
fullpath = stream(ARG(1), 'C', 'QUERY EXISTS')
IF fullpath = "" THEN
DO
  SAY ARG(1) "missing -- aborting"
  exit 1
END

return

/* CompileToLib <source> <library>
 *
 * Compile C source file into object file, then put that object file into
 * the library.  If either of these steps fail, then abort the script.
 *
 * CC, CCOPTS, and ARCHIVE are REXX variables defined at the top
 * of the script.
 */

CompileToLib:

/* Check to make sure the symbols we require are defined  */
/* NOTE: commas at end of line to continue onto next line */
IF (symbol('CC') \= 'VAR') | (symbol('CCOPTS') \= 'VAR') |,
   (symbol('ARCHIVE') \= 'VAR') THEN
DO
  SAY "One of the required symbols is not defined."
  SAY "CC is a " symbol('CC')
  SAY "CCOPTS is a " symbol('CCOPTS')
  SAY "ARCHIVE is a " symbol('ARCHIVE')
  EXIT 99
END

PARSE ARG SOURCE LIBRARY

/* Figure out the filename for the source file */
IF (right(SOURCE, 2) = '.c') | (right(SOURCE, 2 ) = '.C') THEN
DO
  sourcefile   = SOURCE
  basefilename = left(SOURCE, (length(SOURCE) - 2))
END
ELSE
DO
  sourcefile   = SOURCE'.c'
  basefilename = SOURCE
END

/* Join the strings together to get object filename */
objectfile   = basefilename''EOBJ

IF (right(LIBRARY, LIBLEN) = ELIB) | (right(LIBRARY, LIBLEN) = ELIBU) THEN
    libraryfile = LIBRARY
ELSE
    libraryfile = LIBRARY''ELIB

IF verbose = 1 THEN SAY "Compiling "sourcefile"..."

CC CCOPTS '-c' sourcefile
IF (RC > 0) THEN exit 2

IF verbose = 1 
   THEN SAY "Replacing object "objectfile" in library "libraryfile"..."

ARCHIVE libraryfile objectfile
IF (RC > 0) then exit 3
'del' objectfile '>nul'
return

/* CompileToObj <source>
 *
 * Compile C source file into object file.  If this fails, abort the script.
 * CC and CCOPTS are REXX variables defined at the top of the script.
 */

CompileToObj:

/* Check to make sure the symbols we require are defined */
/* NOTE: commas at end of line to continue onto next line */
IF (symbol('CC') \= 'VAR') | (symbol('CCOPTS') \= 'VAR') THEN
DO
   SAY "One of the required symbols is not defined."
   SAY "CC is a " symbol('CC')
   SAY "CCOPTS is a " symbol('CCOPTS')
   EXIT 99
END

/* Figure out the filename for the source file */
IF (right(ARG(1), 2) = '.c') | (right(ARG(1), 2 ) = '.C') THEN
DO
   sourcefile   = ARG(1)
   basefilename = left(ARG(1), (length(ARG(1)) - 2))
END
ELSE
DO
   sourcefile   = ARG(1)'.c'
   basefilename = ARG(1)
END

/* Join the strings together */
objectfile   = basefilename''EOBJ

IF verbose = 1 THEN SAY "Compiling "sourcefile"..."

CC CCOPTS '-c' sourcefile
IF (RC > 0) THEN exit 2
return

/* ArchiveObj <object> <library>
 *
 * Archive the object specified into the library.
 * ARCHIVE is a REXX variable defined at the top of the script.
 */

ArchiveObj:

/* Check to make sure the symbols we require are defined  */
/* NOTE: commas at end of line to continue onto next line */
IF (symbol('ARCHIVE') \= 'VAR') THEN
DO
  SAY "One of the required symbols is not defined."
  SAY "ARCHIVE is a " symbol('ARCHIVE')
  EXIT 99
END

PARSE ARG object library

/* Figure out the filename for the source file */
IF (right(object, 2) = '.c') | (right(object, 2) = '.C') |,
   (right(object, OBJLEN) = EOBJ) | (right(object, OBJLEN) = EOBJU) THEN
  basefilename = left(object, (length(object) - OBJLEN))
ELSE
  basefilename = object

/* Join the strings together to get object filename */
objectfile = basefilename''EOBJ

IF (right(library, LIBLEN) = ELIB) | (right(library, LIBLEN) = ELIBU) THEN
    libraryfile = library
ELSE
    libraryfile = library''ELIB

IF verbose = 1
  THEN SAY "Replacing object "objectfile" in library "libraryfile"..."

ARCHIVE libraryfile objectfile
IF (RC > 0) then exit 3
return

/* LinkProgram <name>
 *
 * Link the object file name.OBJ to get name.EXE.  If the link fails, exit.
 *
 * CC, CCOPTS and LIBRARIES are REXX variables defined at the top
 * of the script.
 */

LinkProgram:

/* Check to make sure the symbols we require are defined */
/* NOTE: commas at end of line to continue onto next one */
IF (symbol('CC') \= 'VAR') | (symbol('CCOPTS') \= 'VAR') |,
   (symbol('LIBRARIES') \= 'VAR') THEN
DO
  SAY "One of the required symbols is not defined."
  EXIT 99
END

basefilename = ARG(1)
IF (right(basefilename, 4) = '.exe') | (right(object, 4) = '.EXE') THEN
  basefilename = left(basefilename, (length(basefilename) - 4))

IF (right(basefilename, 2) = '.c')   | (right(basefilename, 2) = '.C') THEN
  basefilename = left(basefilename, (length(basefilename) - 2))

IF (right(basefilename, OBJLEN) = EOBJ) |,
   (right(basefilename, OBJLEN) = EOBJU) THEN
  basefilename = left(basefilename, (length(basefilename) - OBJLEN))

/* Join the strings together */
/* If we are linking with -Zexe, then the output filename shouldn't have     */
/* ".exe" extension, because this will be added automatically.               */
objectfile   = basefilename''EOBJ
IF (POS("-Zexe", CCOPTS) > 0) THEN
  executable = basefilename
ELSE
  executable = basefilename'.exe'

/* Figure out the libraries to link against */
locallibraries = '-L.'

CALL SysFileTree '*'ELIB, 'files.', 'FO'
DO i = 1 TO files.0
  libfilename = FILESPEC('name', files.i)
  IF libfilename \= '' THEN
  DO
    libfilename = STRIP(libfilename)      /* CMD.EXE leaves trailing spaces*/
    IF (right(libfilename, LIBLEN) = ELIB) |,
       (right(libfilename, LIBLEN) = ELIBU) THEN
        libraryfile = left(libfilename, (length(libfilename) - LIBLEN))
    ELSE
        libraryfile = libfilename

    locallibraries = locallibraries '-l'libraryfile
  END
END

IF verbose = 1 THEN SAY "Linking "executable"..."

CC LINKOPTS '-o' executable objectfile locallibraries LIBRARIES
IF (RC > 0) THEN exit 2
return

/* Find a local library to add ourselves into
 *
 * Strategy is: CCLIBNAME value if set, otherwise look for lib*.a
 */

FindLocalLib:
  locallib = value('CCLIBNAME',,'OS2ENVIRONMENT')
  IF (locallib = '') THEN
  DO
    /* Search for a local library to use */
    CALL SysFileTree '*'ELIB, 'files.', 'FO'
    IF files.0 > 0 THEN
      locallib = FILESPEC('name', files.1)
  END

  IF (locallib = '') THEN locallib = 'libany'ELIB

RETURN locallib

/* End of File */

